///////////////////////////////////////////////////////////////////////////////
//	Filename:      QAWBatAgentApp.cpp
//	Creation Date: 6/28/2006
//	Description:
//    This application will update the .trjob file when associated with a QAWizard Batch file. 
//
//	Copyright  1996-2006 Seapine Software, Inc.
//	All contents of this file are considered Seapine Software proprietary.
///////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

using namespace std;

///////////////////////////////////////////////////////////////////////////////
//    TRJob File Key Values
///////////////////////////////////////////////////////////////////////////////
const string kTRJobKey_ProjectName = "ProjectName:";
const string kTRJobKey_ProjectID = "ProjectID:";
const string kTRJobKey_TestRunName = "TestRunName:";
const string kTRJobKey_TestRunID = "TestRunID:";
const string kTRJobKey_ScriptName = "ScriptName:";
const string kTRJobKey_ScriptType = "ScriptType:";
const string kTRJobKey_ScriptID = "ScriptID:";
const string kTRJobKey_StartedOn = "StartedOn:";
const string kTRJobKey_CompletedOn = "CompletedOn:";
const string  kTRJobKey_Status = "Status:";
const string kTRJobKey_MainResultFile = "MainResultFile:";

///////////////////////////////////////////////////////////////////////////////
//   TRJob File Status Values
///////////////////////////////////////////////////////////////////////////////
const string kTRJobStatus_Scheduled  = "Scheduled";
const string kTRJobStatus_Started  = "Started";
const string kTRJobStatus_Passed  = "Passed";
const string kTRJobStatus_Failed  =  "Failed";
const string kTRJobStatus_Indeterminate = "Indeterminate";

///////////////////////////////////////////////////////////////////////////////
//    The file location strings
///////////////////////////////////////////////////////////////////////////////
string scriptFileLocation;
string trJobFileLocation;

// NOTE: The two following functions are helpers for 
// editing the function FormatDateTime. These provide
// easy ways to convert the dates back and forth. 

//------------------------------------------------------------------------------
// Function: char2string
//    Converts a char, or an array of chars to a string
//
// Parameters:
//    c - The char or char array to convert
//
// Returns:
//    string - The converted string object
//------------------------------------------------------------------------------
string char2string( char* c ) {
 ostringstream buffer;
 buffer << c;
 return buffer.str();
}

//------------------------------------------------------------------------------
// Function: int2string
//    Converts an integer to a string
//
// Parameters:
//    i - An integer
//
// Returns:
//     string - The string representation of the integer. 
//------------------------------------------------------------------------------
string int2string( int i ) {
 ostringstream buffer;
 buffer << i;
 return buffer.str();
}

//------------------------------------------------------------------------------
// Function: SplitString
//   Splits the string into parts based on the passed delimiter
//
// Parameters:
//    str - The string to split
//    delim - What to split it by
//
// Returns:
//     vector<string> - The split string as a vector
//------------------------------------------------------------------------------
vector<string> SplitString( string str, string delim )
{
   string::size_type cutPos;
   vector<string> toReturn;
   while( ( cutPos = str.find_first_of( delim ) ) != str.npos )
   {
      if(cutPos > 0)
      {
         toReturn.push_back( str.substr( 0,cutPos ) );
      }
   str = str.substr( cutPos + 1 );
   }
   if( str.length() > 0 )
   {
      toReturn.push_back( str );   
   }
   return toReturn;
}

//------------------------------------------------------------------------------
// Function: TrimString
//    Removes trailing and leading spaces from a string
//
// Parameters:
//    s - The string to trim
//------------------------------------------------------------------------------
void TrimString( string& s )
{
   string::size_type pos = s.find_last_not_of( ' ' );
  if( pos != string::npos ) {
    s.erase( pos + 1 );
    pos = s.find_first_not_of( ' ' );
    if( pos != string::npos ) s.erase( 0, pos );
  }
  else s.erase( s.begin(), s.end() );
}

//------------------------------------------------------------------------------
// Function: FormatDateTime
//    Formats the system date and time as one string
//    in 12 hour format
//
// Returns:
//    string - The formatted date and time
//------------------------------------------------------------------------------
string FormatDateTime()
{
    char date[9];
    char time[9];
    _strdate( date );
    _strtime( time );
   string toReturn;
   string theDate = char2string( date );
   string theTime = char2string( time );

   toReturn = theDate + " " + theTime;

   return toReturn;
}

//------------------------------------------------------------------------------
// Function: DoesFileExist
//    Checks if the passed in file exists
//
// Parameters:
//    filename - The name of the file
//
// Returns:
//    bool - Returns true if the file exists, false if not 
//------------------------------------------------------------------------------
bool DoesFileExist(string filename)
{
   bool toReturn = false;
   ifstream test( filename.c_str() );
   
   if ( test )
      toReturn = true;
   
   test.close();
   return toReturn;
}

//------------------------------------------------------------------------------
// Function: OpenFileReadAndWriteTempStreams
//    Opens the file streams for reading and writing
//
// Parameters:
//    filename - The filename of the original file
//    in - The ifstream object to read the original file
//    out - The ofstream object to write to the temporary file
//
// Returns:
//    bool - Returns true if opening the streams was
//    successful , false otherwise.
//------------------------------------------------------------------------------
bool OpenFileReadAndWriteTempStreams( string filename, ifstream& in, ofstream& out )
{
   bool toReturn = true;
   string tempFileName = filename + ".tmp";
   in.open( filename.c_str() );
   out.open( tempFileName.c_str() );
   if ( !in || !out )
      toReturn = false;

   return toReturn;
}

//------------------------------------------------------------------------------
// Function: ReplaceTRJobFileWithTemp
//    Replaces the original TRJobFile with the temporary
//    one that we updated. 
//
// Returns:
//    bool - Returns true if the file was successfully copied
//------------------------------------------------------------------------------
bool ReplaceTRJobFileWithTemp()
{
   bool toReturn = false;
   string tempFileName = trJobFileLocation + ".tmp";
   if ( remove( trJobFileLocation.c_str() ) == 0 )
      if ( rename( tempFileName.c_str(),trJobFileLocation.c_str() ) == 0 )
         toReturn = true;
   return toReturn;
}

//------------------------------------------------------------------------------
// Function: UpdateTRJobFileOnStart
//   Updates the TRJob file's status and StartedOn info
//------------------------------------------------------------------------------
void UpdateTRJobFileOnStart()
{
   string trLine;
   ifstream infile;
   ofstream outfile;
   if ( OpenFileReadAndWriteTempStreams( trJobFileLocation, infile, outfile ) )
   {
      while ( getline( infile, trLine ) )
      {
         TrimString(trLine);
         vector<string> trLineVector = SplitString( trLine," " );

         if ( trLineVector[0] == kTRJobKey_StartedOn )
            trLine = trLine + " " + FormatDateTime();
         if ( trLineVector[0] == kTRJobKey_Status )
            trLine = kTRJobKey_Status + " " + kTRJobStatus_Started;
            
         outfile << trLine << " " << endl;
      }
      infile.close();
      outfile.close();
      ReplaceTRJobFileWithTemp();
   }
}

//------------------------------------------------------------------------------
// Function: ReplaceTRJobFileOnCompletion
//    Updates the TRJob file's status, CompletedOn, and
//    MainResultFile on completion.
//
// Parameters:
//    result - The result of executing the QAWizzard
//                batch file. 
//------------------------------------------------------------------------------
void UpdateTRJobFileOnCompletion( int result )
{
   string trLine;
   ifstream infile;
   ofstream outfile;
   if ( OpenFileReadAndWriteTempStreams( trJobFileLocation, infile, outfile ) ) 
   {
      while ( getline( infile, trLine ) )
      {
         TrimString(trLine);
         vector<string> trLineVector = SplitString( trLine, " " );

         if ( trLineVector[0] == kTRJobKey_CompletedOn )
            trLine = trLine + " " + FormatDateTime();
         if ( trLineVector[0] == kTRJobKey_Status )
         {
           switch( result )
           {
           case 0:
              trLine = kTRJobKey_Status + " " + kTRJobStatus_Passed;
              break;
           case 1:
              trLine = kTRJobKey_Status + " " + kTRJobStatus_Failed;
              break;
           default:
              trLine = kTRJobKey_Status + " " + kTRJobStatus_Indeterminate;
           }
         }
         if ( trLineVector[0] == kTRJobKey_MainResultFile )
            trLine = trLine + " .\\BatchReport0\\Report.xml ";

         outfile << trLine << " " << endl;
      }
      infile.close();
      outfile.close();
      ReplaceTRJobFileWithTemp();
   }
}

///////////////////////////////////////////////////////////////////////////////
//                                  MAIN
///////////////////////////////////////////////////////////////////////////////
int _tmain( int argc, _TCHAR* argv[] )
{ 
   if ( argc > 1 )
   {
      // Put back together the command line arguments as c++ splits them at white space
      string tempArgs;
      for ( int i = 1; i < argc; i++ )
         tempArgs = tempArgs + argv[i] + " ";

      TrimString( tempArgs );
      scriptFileLocation = tempArgs;
      
      // Check if the file exists
      if ( !DoesFileExist( scriptFileLocation.c_str() ) )
         return 1;
     
      // Build the path to the trjob file
      vector<string> scriptFileLocationVec = SplitString( scriptFileLocation, "." );
      for ( vector<string>::size_type i = 0; i < scriptFileLocationVec.size() - 1; i++ )
         trJobFileLocation = trJobFileLocation + scriptFileLocationVec[i] + ".";

      trJobFileLocation = trJobFileLocation + "trjob";
      
      // Update TRJob file with start info
      UpdateTRJobFileOnStart();

      // Add quotes to the batch file in case there are spaces
      string scriptFileLocationWithQuotes = "\"" + scriptFileLocation + "\"";
      
      // Run the script
      int runResult = system( scriptFileLocationWithQuotes.c_str() );
      
      // Update TRJob file with completion info
      UpdateTRJobFileOnCompletion( runResult );
      
      return 0;
   }
   return 1;
}
